Imports System
Imports System.Xml
Imports System.Data
Imports System.Text
Imports System.Drawing
Imports System.Threading
Imports System.Reflection
Imports System.Collections
Imports System.Globalization
Imports System.ComponentModel
Imports Microsoft.Win32
Imports Microsoft.PointOfService

Public Class CCReader
    Private csAccount As String
    Private csExpirationDate As String
    Private csTitle As String
    Private csFirstName As String
    Private csLastName As String
    Private csMiddleInitial As String
    Private ciServiceCode As Integer
    Private csSuffix As String

    Private coPosCommon As posCommon
    Private coPosExplorer As posExplorer
    Private ccDevices As DeviceCollection
    Private coDeviceLD As DeviceInfo
    Private coDeviceMSR As DeviceInfo

    Friend WithEvents coMsr As Msr
    Friend WithEvents coLD As LineDisplay

    Public Event CreditCardScanned(ByVal cbValue As Boolean)

    Sub New()
        csAccount = ""
        csExpirationDate = ""
        csTitle = ""
        csFirstName = ""
        csLastName = ""
        csMiddleInitial = ""
        ciServiceCode = 0
        csSuffix = ""
    End Sub

    ' Miscellanous Functions & Subroutines -----------------

    Private Function GetDeviceDisplayName(ByVal device As DeviceInfo) As String
        Dim name As String = CombineNames(device.LogicalNames)
        If name.Length = 0 Then
            name = device.ServiceObjectName
            If name.Length = 0 Then
                name = device.Description
            End If
        End If
        Return name
    End Function

    Private Shared Function CombineNames(ByVal names As String()) As String
        Dim s As String = ""
        For Each name As String In names
            If s.Length > 0 Then
                s += ";"c
            End If
            s += name
        Next
        Return s
    End Function

    Private Function GetRegistryValue() As Boolean
        Dim regKey As RegistryKey
        Dim regValue As String = ""
        Try
            regKey = Registry.LocalMachine.OpenSubKey("Software\POSfor.NET\Setup", False)
            regValue = CStr(regKey.GetValue("InstallPath"))
        Catch ex As Exception
            ' do nothing
        End Try
        If regValue.Length > 0 Then
            Return True
        Else
            Return False
        End If
    End Function

    Private Function GetString(ByVal b As Byte()) As String
        If b Is Nothing Then
            Return ""
        End If

        Dim sb As New StringBuilder(b.Length)
        For i As Integer = 0 To b.Length - 1
            sb.Append(ChrW(b(i)))
        Next
        Return sb.ToString()
    End Function

    Public Function IsDevicePresent() As Boolean
        Dim loDevice As DeviceInfo

        If GetRegistryValue() = True Then
            Try
                coPosExplorer = New PosExplorer
                ccDevices = coPosExplorer.GetDevices(DirectCast([Enum].Parse(GetType(DeviceCompatibilities), "OposAndCompatibilityLevel1", False), DeviceCompatibilities))
                For Each loDevice In ccDevices
                    ' Create Line Display Object ----------------------------------------------------
                    If loDevice.Type = "LineDisplay" And loDevice.ServiceObjectName = "Ing6XXX" Then
                        coDeviceLD = loDevice
                    End If

                    ' Create MSR Object -------------------------------------------------------------
                    If loDevice.Type = "Msr" And loDevice.ServiceObjectName = "Ing6XXX" Then
                        coDeviceMSR = loDevice
                    End If
                Next
                If Not coDeviceLD Is Nothing And Not coDeviceMSR Is Nothing Then
                    Return True
                Else
                    Return False
                End If
            Catch ex As Exception
                Return False
            End Try
        End If
    End Function

    ' Main Functions & Subroutines -------------------------

    Public Function ShowLDMessage(ByVal DisplayText As String) As Boolean
        Try
            coLD.Open()
            coLD.Claim(30)
            coLD.DeviceEnabled = True
            coLD.ClearText()
            coLD.DisplayText(DisplayText, DisplayTextMode.Normal)
            coLD.DeviceEnabled = False
            coLD.Release()
            coLD.Close()
            Return True
        Catch ex As Exception
            Return False
        End Try
    End Function

    Public Function StartSwipe() As Boolean
        Try
            coPosCommon = DirectCast(coPosExplorer.CreateInstance(coDeviceLD), PosCommon)
            coLD = DirectCast(coPosCommon, LineDisplay)

            ' Set Line Display Text ---------------------------
            ShowLDMessage("Scan Card: ")

            ' Set MSR Settings --------------------------------
            coPosCommon = DirectCast(coPosExplorer.CreateInstance(coDeviceMSR), PosCommon)
            coMsr = DirectCast(coPosCommon, Msr)

            coMsr.Open()
            coMsr.Claim(30)
            coMsr.DeviceEnabled = True
            coMsr.DataEventEnabled = True
            Return True
        Catch ex As Exception
            Return False
        End Try
    End Function

    Public Sub DataObtained(ByVal source As Object, ByVal d As DataEventArgs) Handles coMsr.DataEvent
        Dim str As String = d.ToString() + "" & Chr(13) & "" & Chr(10) & ""
        Dim imsr As Msr = DirectCast(coPosCommon, Msr)
        'Dim ccdata1 As New CCData

        csAccount = imsr.AccountNumber
        csExpirationDate = imsr.ExpirationDate
        csTitle = imsr.Title
        csFirstName = imsr.FirstName
        csLastName = imsr.Surname
        csMiddleInitial = imsr.MiddleInitial
        ciServiceCode = CInt(imsr.ServiceCode)
        csSuffix = imsr.Suffix

        Try
            coMsr.DataEventEnabled = False
            coMsr.DeviceEnabled = False
            coMsr.Release()
            'coMsr.Close()
            'coMsr = Nothing

            ' Set Line Display Text ---------------------------
            ShowLDMessage("Thank You")
        Catch ex As Exception
            Throw ex
        End Try
        RaiseEvent CreditCardScanned(True)
    End Sub

    ' Properties -------------------------------------------

    Public ReadOnly Property Account() As String
        Get
            Return csAccount
        End Get
    End Property
    Public ReadOnly Property ExpirationDate() As String
        Get
            Return csExpirationDate
        End Get
    End Property
    Public ReadOnly Property Title() As String
        Get
            Return csTitle
        End Get
    End Property
    Public ReadOnly Property FirstName() As String
        Get
            Return csFirstName
        End Get
    End Property
    Public ReadOnly Property LastName() As String
        Get
            Return csLastName
        End Get
    End Property
    Public ReadOnly Property MiddleInitial() As String
        Get
            Return csMiddleInitial
        End Get
    End Property
    Public ReadOnly Property ServiceCode() As Integer
        Get
            Return ciServiceCode
        End Get
    End Property
    Public ReadOnly Property Suffix() As String
        Get
            Return csSuffix
        End Get
    End Property

End Class
